// Find Hard Links Utility.
//
// Copyright (c) 2000 Mike Woodring
// http://www.bearcanyon.com
//
#pragma warning(disable:4786)

#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include <conio.h>
#include "netconn.h"
#include "recurse.h"
#include "_BldNum.h"
#include "..\include\ErrorMsg.h"
#include <list>
#include <string>
using namespace std;

#define TEST_CMDLINE_PARSING    0

#ifdef _DEBUG
#define ALWAYS_MATCH            1
#else
#define ALWAYS_MATCH            0
#endif

#ifdef UNICODE
    #define ustring wstring
#else
    #define ustring string
#endif

struct SEARCHRESULTS
{
    LARGE_INTEGER   liRefFileIndex;     // Reference file identifier.
    list<ustring *> listLinkedFiles;    // List of linked files with same identifier.
    TCHAR           *pszUNCPath;        // 0 if not specified.

    SEARCHRESULTS()
        : pszUNCPath(0)
    {
    }

    ~SEARCHRESULTS()
    {
        list<ustring *>::iterator it;

        for( it = listLinkedFiles.begin(); it != listLinkedFiles.end(); ++it )
        {
            delete (*it);
        }

        listLinkedFiles.clear();
    }
};

BOOL WINAPI OnEachDir( void *pvContext )
{
    SEARCHRESULTS *pSearchResults = (SEARCHRESULTS *)pvContext;

    WIN32_FIND_DATA FindData;
    HANDLE          hFind;

    hFind = FindFirstFile(__TEXT("*.*"), &FindData);

    if( hFind != INVALID_HANDLE_VALUE )
    {
        BOOL fMoreFiles;

        do
        {
            if( 0 == (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) )
            {
                // This is a file, not a directory.  Check to see if it's a link
                // to the reference file.
                //
                HANDLE hFile = CreateFile( FindData.cFileName, GENERIC_READ,
                                           FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING,
                                           FILE_ATTRIBUTE_NORMAL, 0 );

                if( hFile != INVALID_HANDLE_VALUE )
                {
                    BY_HANDLE_FILE_INFORMATION FileInfo = {0};

                    GetFileInformationByHandle(hFile, &FileInfo);
                    CloseHandle(hFile);

#if ALWAYS_MATCH
                    if( TRUE )
#else
                    if( (FileInfo.nFileIndexLow == pSearchResults->liRefFileIndex.LowPart) &&
                        (FileInfo.nFileIndexHigh == pSearchResults->liRefFileIndex.HighPart) )
#endif
                    {
                        TCHAR szFileName[MAX_PATH];
                        TCHAR szCurDir[MAX_PATH];

                        GetCurrentDirectory(MAX_PATH, szCurDir);

                        if( szCurDir[_tcslen(szCurDir) - 1] != '\\' )
                        {
                            _tcscat(szCurDir, __TEXT("\\"));
                        }

                        if( pSearchResults->pszUNCPath != 0 )
                        {
                            TCHAR szServer[MAX_PATH+1];
                            TCHAR szSubDir[MAX_PATH+1];

                            _NetSplitUNCPath(pSearchResults->pszUNCPath, szServer, szSubDir);
                            _stprintf( szFileName, __TEXT("%s\\%s%s"),
                                       szServer, &szCurDir[3], FindData.cFileName );
                        }
                        else
                        {
                            _stprintf( szFileName, __TEXT("%s%s"),
                                       szCurDir, FindData.cFileName );
                        }

                        pSearchResults->listLinkedFiles.push_back( new ustring(szFileName) );
                    }
                }
            }

            fMoreFiles = FindNextFile(hFind, &FindData);
        }
        while( fMoreFiles );

        FindClose(hFind);
    }

    return(TRUE);
}

void ShowSearchResults( SEARCHRESULTS *pResults )
{
    list<ustring *>::iterator it;

    for( it = pResults->listLinkedFiles.begin(); it != pResults->listLinkedFiles.end(); ++it )
    {
        _tprintf(__TEXT("%s\n"), (*it)->c_str());
    }
}

void ShowUsage( void )
{
    _tprintf(
        __TEXT("\n")
        __TEXT("Find Hard Links Utility v%d.%d.%d.%d\n")
        __TEXT("Mike Woodring\n")
        __TEXT("http://www.bearcanyon.com\n")
        __TEXT("\n")
        __TEXT("Command line:\n")
        __TEXT("    flinks /f:file [/d:directory]\n")
        __TEXT("\n")
        __TEXT("Where:\n")
        __TEXT("    file        Identifies the reference file to find links to.\n")
        __TEXT("    directory   Identifies the starting directory location.\n")
        __TEXT("\n"),
        FILE_VERSION0,
        FILE_VERSION1,
        FILE_VERSION2,
        BUILD_NUMBER
    );
}

extern "C" int _tmain( int cArgs, TCHAR *pszArgs[] )
{
    // flinks /f:reference_file [/d:directory]
    //
    if( cArgs >= 2 )
    {
        TCHAR  *pszRefFile = 0;
        TCHAR  *pszDir = 0;
        TCHAR   szCurDir[MAX_PATH + 1];

        for( int iArg = 1; iArg < cArgs; iArg++ )
        {
            #if TEST_CMDLINE_PARSING
            _tprintf( __TEXT("recurse arg[%d] == %s\n"),
                      iArg, pszArgs[iArg] );
            getch();
            #endif

            if( (_tcsnicmp(pszArgs[iArg], __TEXT("/d:"), 3) == 0) ||
                (_tcsnicmp(pszArgs[iArg], __TEXT("-d:"), 3) == 0) )
            {
                pszDir = pszArgs[iArg] + 3;
            }
            else if( (_tcsnicmp(pszArgs[iArg], __TEXT("/f:"), 3) == 0) ||
                     (_tcsnicmp(pszArgs[iArg], __TEXT("-f:"), 3) == 0) )
            {
                pszRefFile = pszArgs[iArg] + 3;
            }
        }

        if( pszRefFile )
        {
            if( !pszDir )
            {
                GetCurrentDirectory(MAX_PATH, szCurDir);
                pszDir = szCurDir;
            }

            HANDLE hRefFile = CreateFile( pszRefFile, GENERIC_READ, 0, 0, OPEN_EXISTING,
                                          FILE_ATTRIBUTE_NORMAL, 0 );

            if( INVALID_HANDLE_VALUE == hRefFile )
            {
                _tprintf(__TEXT("Could not open reference file %s.\n"), pszRefFile);
                return(0);
            }

            BY_HANDLE_FILE_INFORMATION FileInfo = {0};

            GetFileInformationByHandle(hRefFile, &FileInfo);
            CloseHandle(hRefFile);

#if ALWAYS_MATCH
            if( TRUE )
#else
            if( FileInfo.nNumberOfLinks > 1 )
#endif
            {
                _tprintf( __TEXT("%s has %d links:\n"), 
                          pszRefFile, (FileInfo.nNumberOfLinks - 1) );

                SEARCHRESULTS SearchResults;

                SearchResults.liRefFileIndex.LowPart = FileInfo.nFileIndexLow;
                SearchResults.liRefFileIndex.HighPart = FileInfo.nFileIndexHigh;

                try
                {
                    // Go do it, starting with the current directory.
                    //
                    if( _NetIsUNCPath(pszDir) )
                    {
                        TCHAR szServer[MAX_PATH+1];
                        TCHAR szSubDir[MAX_PATH+1];
                        TCHAR szDrive[3];

                        _NetSplitUNCPath(pszDir, szServer, szSubDir);

                        NET_API_STATUS Status;

                        Status = _NetAddConnection(szServer, 0, 0, szDrive);

                        if( NERR_Success == Status )
                        {
                            TCHAR szLocalPath[MAX_PATH+1];

                            wsprintf( szLocalPath, __TEXT("%s%s"),
                                      szDrive, szSubDir );

                            #if TEST_CMDLINE_PARSING
                                _tprintf( __TEXT("file  : %s\n")
                                          __TEXT("dir   : %s\n"),
                                          pszRefFile,
                                          szLocalPath );
                                getch();
                            #else
                                SearchResults.pszUNCPath = pszDir;
                                Recurse(szLocalPath, OnEachDir, &SearchResults);
                            #endif

                            _NetDropConnection(szDrive);
                        }
                        else
                        {
                            ErrorMessage(__TEXT("_NetAddConnection"), Status);
                        }
                    }
                    else
                    {
                        #if TEST_CMDLINE_PARSING
                                _tprintf( __TEXT("file  : %s\n")
                                          __TEXT("dir   : %s\n"),
                                          pszRefFile,
                                          pszDir );
                            getch();
                        #else
                            Recurse(pszDir, OnEachDir, &SearchResults);
                        #endif
                    }

                    ShowSearchResults(&SearchResults);
                }
                catch( ... )
                {
                    _tprintf( __TEXT("\nFatal error encountered while searching for links to \"%s\"."),
                              pszRefFile );
                }
            }
            else
            {
                _tprintf(__TEXT("There are no links to %s.\n"), pszRefFile);
            }
        }
        else
        {
            _tprintf(__TEXT("No reference file entered\n"));
        }
    }
    else
    {
        ShowUsage();
    }

    return(0);
}
