#include <windows.h>
#include <lmaccess.h>
#include <lmuse.h>
#include <tchar.h>
#include <stdio.h>
#include <conio.h>
#include "netconn.h"

// _NetAddConnection
//
// Wrapper around NetUesAdd to map a network share to an
// available driver letter.  This funciton start at Z: and loops
// down through drive letters until an available drive mapping is
// found or a failure occurs.  The drive letter will be returned
// as a string of the form "D:" via szDriveLetter on
// successful return from this function.  If this function succeeds,
// NERR_Success will be returned.
//
// pszRemoteRes must be in the following format: \\servername\sharename.
//
// To work with \\servername\sharename\dir\dir\file, you'll have to map
// \\servername\sharename using this function to get a drive letter, then
// use d:\dir\dir\file to access the particular resource within the share.
//
NET_API_STATUS _NetAddConnection( IN LPTSTR pszRemoteRes, IN LPCSTR pszUserName,
                                  IN LPCSTR pszPassword, OUT TCHAR szDriveLetter[3] )
{
    NET_API_STATUS  Status;
    USE_INFO_2      ui;
    TCHAR           szDrive[3];
    TCHAR           chDrive;
    BOOL            fContinue = TRUE;
    DWORD           nIndex = -1;

    for( chDrive = 'z'; (chDrive > 'c') && fContinue; chDrive-- )
    {
        wsprintf(szDrive, __TEXT("%c:"), chDrive);
        ZeroMemory(&ui, sizeof(ui));

        ui.ui2_local = szDrive;
        ui.ui2_remote = pszRemoteRes;
        ui.ui2_asg_type = USE_DISKDEV;
        ui.ui2_usecount = 1;

        Status = NetUseAdd(0, 2, (LPBYTE)&ui, &nIndex);

        switch( Status )
        {
            case ERROR_ALREADY_ASSIGNED:
            case ERROR_DEVICE_ALREADY_REMEMBERED:
                // Try again on a different drive letter.
                //
            break;

            case NERR_Success:
                // Successfully mapped chDrive.
                //
                fContinue = FALSE;
                wsprintf(szDriveLetter, __TEXT("%c:"), chDrive);
            break;

            default:
                // A "real" error occurred that makes it pointless to
                // try again (eg: ERROR_NO_NETWORK).
                //
                fContinue = FALSE;
            break;
        }
    }

    return(Status);
}

// _NetDropConnection
//
// Undoes the work of _NetAddConnection.
//
void _NetDropConnection( IN TCHAR szDriveLetter[3] )
{
    NetUseDel(0, szDriveLetter, USE_FORCE);
}

// _NetIsUNCPath
//
BOOL _NetIsUNCPath( IN TCHAR *pszPath )
{
    if( (_tcslen(pszPath) >= 2) &&
        (pszPath[0] == '\\') &&
        (pszPath[1] == '\\') )
    {
        return(TRUE);
    }

    return(FALSE);
}

// _NetSplitUNCPath
//
//  IN:  \\servername\sharename[\dir\dir\file]
//  OUT: \\servername\sharename
//  OUT: \dir\dir\file
//
BOOL _NetSplitUNCPath( IN TCHAR *pszPath, OUT TCHAR *pszServerShare,
                       OUT TCHAR *pszFilePath )
{
    pszServerShare[0] = 0;
    pszFilePath[0] = 0;

    // Verify that pszPath is a UNC pathname.
    //
    if( !_NetIsUNCPath(pszPath) )
    {
        return(FALSE);
    }

    BOOL    fSplitOk = FALSE;
    TCHAR  *psz;
    TCHAR  *pszFilePart = 0;

    // Locate the end of \\servername\sharename, copying it
    // out to the caller.
    //
    psz = _tcschr(pszPath+2, '\\');

    if( psz )
    {
        // Found end of \\servername\, check for sharename\ part.
        //
        int cb = 0;

        psz = _tcschr(psz+1, '\\');

        if( psz )
        {
            // Found sharename\ part, so a filepart follows.
            //
            cb = (DWORD)psz - (DWORD)pszPath;
            pszFilePart = psz;
        }
        else
        {
            cb = _tcslen(pszPath) * sizeof(TCHAR);
        }

        memcpy(pszServerShare, pszPath, cb);
        
        pszServerShare[cb / sizeof(TCHAR)] = 0;

        if( pszFilePart )
        {
            _tcscpy(pszFilePath, pszFilePart);
        }

        fSplitOk = TRUE;
    }

    return(fSplitOk);
}
