#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include <conio.h>
#include "netconn.h"
#include "..\include\ErrorMsg.h"
#include "recurse.h"

// IsSubDirectory
//
// Tests the file information specified in WIN32_FIND_DATA for
// the directory attribute.  Ignores directories with the name
// "." or "..".
//
BOOL IsSubDirectory( WIN32_FIND_DATA *pFindData )
{
    BOOL fIsSubDir = FALSE;

    if(
        ((pFindData->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) &&
        (_tcsicmp(__TEXT("."), pFindData->cFileName) != 0) &&
        (_tcsicmp(__TEXT(".."), pFindData->cFileName) != 0)
    )
    {
        fIsSubDir = TRUE;
    }

    return(fIsSubDir);
}

// FindNextSubDirectory
//
// Uses FindNextFile to enumerate through files matching the search
// specification.  Returns when WIN32_FIND_DATA contains the next
// file found that has the subdirectory attribute, and is not
// "." or "..".
//
BOOL FindNextSubDirectory( HANDLE hFindHandle, WIN32_FIND_DATA *pFindData )
{
    BOOL fMoreToSearch = TRUE;
    BOOL fFoundSubDir = FALSE;

    do
    {
        fMoreToSearch = FindNextFile(hFindHandle, pFindData);

        if( fMoreToSearch )
        {
            fFoundSubDir = IsSubDirectory(pFindData);
        }
    }
    while( fMoreToSearch && !fFoundSubDir );

    return(fFoundSubDir);
}

// Uses FindFirstFile and FindNextSubDirectory to locate the first
// file matching pszSpec that is has the directory attribute and
// is not "." or "..".
//
HANDLE FindFirstSubDirectory( TCHAR *pszSpec, WIN32_FIND_DATA *pFindData )
{
    HANDLE hFindHandle = INVALID_HANDLE_VALUE;

    hFindHandle = FindFirstFile(pszSpec, pFindData);

    if( hFindHandle != INVALID_HANDLE_VALUE )
    {
        BOOL fFoundSubDir = FALSE;

        fFoundSubDir = IsSubDirectory(pFindData);

        if( !fFoundSubDir )
        {
            fFoundSubDir = FindNextSubDirectory(hFindHandle, pFindData);
        }

        if( !fFoundSubDir )
        {
            FindClose(hFindHandle);
            hFindHandle = INVALID_HANDLE_VALUE;
        }
    }

    return(hFindHandle);
}

// ExecuteCommand
//
// Executes the given command.  Optionally prints out a message indicating
// the command being executed and the directory it's being executed in.
//
void ExecuteCommand( TCHAR *pszCmd, BOOL fQuietMode )
{
    if( !fQuietMode )
    {
        static TCHAR szCurDir[2 * MAX_PATH + 1];
        static TCHAR szBuf[4 * MAX_PATH + 1];

        GetCurrentDirectory(sizeof(szCurDir), szCurDir);

        wsprintf(
            szBuf,
            __TEXT("___________________________________________________________________________\n")
            __TEXT("Executing command : %s\n")
            __TEXT("in directory      : %s\n")
            __TEXT("---------------------------------------------------------------------------\n"),
            pszCmd,
            szCurDir
        );

        _tprintf(szBuf);
    }

    // Do it!
    //
    _tsystem(pszCmd);
}

BOOL Recurse( TCHAR *pszStartDir, PFXNRECURSECALLBACK pfxnForEachDir, void *pvContext )
{
    WIN32_FIND_DATA FindData;
    HANDLE          hFindHandle;
    TCHAR          *pszOriginalDir = new TCHAR[2 * MAX_PATH + 1];
    BOOL            fChangedDir;
    BOOL            fContinue = FALSE;

    // Save off where we're starting at.
    //
    GetCurrentDirectory(2 * MAX_PATH, pszOriginalDir);
    
    // Move to the target directory.
    //
    fChangedDir = SetCurrentDirectory(pszStartDir);

    if( fChangedDir )
    {
        // Process this directory.
        //
        fContinue = pfxnForEachDir(pvContext);

        if( fContinue )
        {
            // Recursively dive into this directory's subdirectories (if any).
            //
            hFindHandle = FindFirstSubDirectory(__TEXT("*.*"), &FindData);

            if( hFindHandle != INVALID_HANDLE_VALUE )
            {
                BOOL fMoreDirs = TRUE;

                while( fContinue && fMoreDirs )
                {
                    fContinue = Recurse(FindData.cFileName, pfxnForEachDir, pvContext);

                    if( fContinue )
                    {
                        fMoreDirs = FindNextSubDirectory(hFindHandle, &FindData);
                    }
                }

                FindClose(hFindHandle);
            }
        }

        // Restore the working directory to where it was when we entered
        // this function.
        //
        SetCurrentDirectory(pszOriginalDir);
    }

    delete [] pszOriginalDir;
    return(fContinue);
}
